@@ -16,6 +16,7 @@
 from pecan import expose, response
 from load_balancers import LoadBalancersController
 from limits import LimitsController
+from protocols import ProtocolsController
 from libra.api.model.responses import Responses
 
 
@@ -25,19 +26,7 @@ class V1Controller(object):
     @expose('json')
     def index(self):
         response.status = 200
-        return Responses.versions
-
-    @expose('json')
-    def protocols(self):
-        """Lists all supported load balancing protocols.
-
-        Url:
-           GET /protocols
-
-        Returns: dict
-        """
-        response.status = 200
-        return Responses.protocols
+        return Responses.v1_1
 
     @expose('json')
     def algorithms(self):
@@ -51,6 +40,7 @@ class V1Controller(object):
         response.status = 200
         return Responses.algorithms
 
-    #pecan uses this controller class for urls that start with /loadbalancers
+    # pecan uses this controller class for urls that start with /loadbalancers
     loadbalancers = LoadBalancersController()
     limits = LimitsController()
+    protocols = ProtocolsController()
