@@ -13,18 +13,22 @@
 # under the License.
 
 import eventlet
+from libra import gear #TODO
+import json
+
 eventlet.monkey_patch()
 import ipaddress
-from libra.common.json_gearman import JSONGearmanClient
 from libra.common.api.lbaas import LoadBalancer, db_session, Device, Node, Vip
-from libra.common.api.lbaas import HealthMonitor
+from libra.common.api.lbaas import HealthMonitor, Counters
 from libra.common.api.lbaas import loadbalancers_devices
+from libra.common.api.mnb import update_mnb
 from libra.openstack.common import log
 from pecan import conf
-
+from time import sleep
 
 LOG = log.getLogger(__name__)
-
+POLL_COUNT = 10
+POLL_SLEEP = 10
 
 gearman_workers = [
     'UPDATE',  # Create/Update a Load Balancer.
@@ -33,11 +37,22 @@ gearman_workers = [
     'DELETE',  # Delete a Load Balancer.
     'DISCOVER',  # Return service discovery information.
     'ARCHIVE',  # Archive LB log files.
-    'STATS',  # Get load balancer statistics.
-    'PING'    # Ping load balancers
+    'METRICS',  # Get load balancer statistics.
+    'STATS'     # Ping load balancers
 ]
 
 
+class DisconnectClient(gear.Client):
+    def handleDisconnect(self, job):
+        job.disconnect = True
+
+
+class DisconnectJob(gear.Job):
+    def __init__(self, name, arguments):
+        super(DisconnectJob, self).__init__(name, arguments)
+        self.disconnect = False
+
+
 def submit_job(job_type, host, data, lbid):
     eventlet.spawn_n(client_job, job_type, str(host), data, lbid)
 
@@ -77,7 +92,7 @@ def client_job(job_type, host, data, lbid):
                         .format(data)
                     )
                     return
-
+                mnb_data = {}
                 if not status:
                     LOG.error(
                         "Giving up vip assign for device {0}".format(data)
@@ -93,10 +108,22 @@ def client_job(job_type, host, data, lbid):
                         filter(LoadBalancer.status != 'DELETED').\
                         all()
                     for lb in lbs:
+                        if lb.status == 'BUILD':
+                            # Only send a create message to MnB if we
+                            # are going from BUILD to ACTIVE. After the
+                            # DB is updated.
+                            mnb_data["lbid"] = lb.id
+                            mnb_data["tenantid"] = lb.tenantid
                         lb.status = 'ACTIVE'
                     device.status = 'ONLINE'
                 session.commit()
 
+                # Send the MnB create if needed
+                if "lbid" in mnb_data:
+                    update_mnb('lbaas.instance.create',
+                               mnb_data["lbid"],
+                               mnb_data["tenantid"])
+
         if job_type == 'REMOVE':
             client.send_remove(data)
         return
@@ -109,19 +136,15 @@ class GearmanClientThread(object):
         self.host = host
         self.lbid = lbid
 
-        server_list = []
+        self.gear_client = DisconnectClient()
+
         for server in conf.gearman.server:
             ghost, gport = server.split(':')
-            server_list.append({'host': ghost,
-                                'port': int(gport),
-                                'keyfile': conf.gearman.ssl_key,
-                                'certfile': conf.gearman.ssl_cert,
-                                'ca_certs': conf.gearman.ssl_ca,
-                                'keepalive': conf.gearman.keepalive,
-                                'keepcnt': conf.gearman.keepcnt,
-                                'keepidle': conf.gearman.keepidle,
-                                'keepintvl': conf.gearman.keepintvl})
-        self.gearman_client = JSONGearmanClient(server_list)
+            self.gear_client.addServer(ghost,
+                                       int(gport),
+                                       conf.gearman.ssl_key,
+                                       conf.gearman.ssl_cert,
+                                       conf.gearman.ssl_ca)
 
     def send_assign(self, data):
         NULL = None  # For pep8
@@ -129,7 +152,7 @@ class GearmanClientThread(object):
             device = session.query(Device).\
                 filter(Device.name == data).first()
             if device is None:
-                self.LOG.error(
+                LOG.error(
                     "VIP assign have been given non existent device {0}"
                     .format(data)
                 )
@@ -154,7 +177,7 @@ class GearmanClientThread(object):
                     filter(Vip.id == self.lbid).first()
                 if vip is None:
                     errmsg = 'Cannot find existing floating IP'
-                    self.LOG.error(
+                    LOG.error(
                         "Failed to assign IP to device {0}"
                         .format(data)
                     )
@@ -176,12 +199,12 @@ class GearmanClientThread(object):
         if status:
             return True
         elif self.lbid:
-            self.LOG.error(
+            LOG.error(
                 "Failed to assign IP {0} to device {1}"
                 .format(ip_str, data)
             )
         else:
-            self.LOG.error(
+            LOG.error(
                 "Failed to assign IP {0} to device {1}"
                 .format(ip_str, data)
             )
@@ -200,7 +223,7 @@ class GearmanClientThread(object):
         }
         ip_int = int(ipaddress.IPv4Address(unicode(self.lbid)))
         for x in xrange(0, 5):
-            self.logger.info(
+            LOG.info(
                 'Attempt to delete IP {0} #{1}'
                 .format(self.lbid, x)
             )
@@ -221,6 +244,9 @@ class GearmanClientThread(object):
             else:
                 session.query(Vip).\
                     filter(Vip.ip == ip_int).delete()
+                counter = session.query(Counters).\
+                    filter(Counters.name == 'vips_deleted').first()
+                counter.value += 1
             session.commit()
 
     def send_delete(self, data):
@@ -286,6 +312,8 @@ class GearmanClientThread(object):
                 )
                 self._set_error(data, response, session)
             lb.status = 'DELETED'
+            tenant_id = lb.tenantid
+
             if count == 0:
                 # Device should never be used again
                 device = session.query(Device).\
@@ -299,8 +327,14 @@ class GearmanClientThread(object):
                 filter(Node.lbid == lb.id).delete()
             session.query(HealthMonitor).\
                 filter(HealthMonitor.lbid == lb.id).delete()
+            counter = session.query(Counters).\
+                filter(Counters.name == 'loadbalancers_deleted').first()
+            counter.value += 1
             session.commit()
 
+            # Notify billing of the LB deletion
+            update_mnb('lbaas.instance.delete', self.lbid, tenant_id)
+
     def _set_error(self, device_id, errmsg, session):
         lbs = session.query(
             LoadBalancer
@@ -317,7 +351,10 @@ class GearmanClientThread(object):
             # and auto-failover
             return
         device.status = 'ERROR'
+        counter = session.query(Counters).\
+            filter(Counters.name == 'loadbalancers_error').first()
         for lb in lbs:
+            counter.value += 1
             lb.status = 'ERROR'
             lb.errmsg = errmsg
 
@@ -347,6 +384,9 @@ class GearmanClientThread(object):
             else:
                 device.errmsg = 'Log archive failed: {0}'.format(response)
             lb.status = 'ACTIVE'
+            counter = session.query(Counters).\
+                filter(Counters.name == 'log_archives').first()
+            counter.value += 1
             session.commit()
 
     def send_update(self, data):
@@ -422,10 +462,22 @@ class GearmanClientThread(object):
                 if monitor.path is not None:
                     monitor_data['path'] = monitor.path
 
+                # All new LBs created since these options were supported
+                # will have default values in the DB. Pre-existing LBs will
+                # not have any values, so we need to check for that.
+                if any([lb.timeout, lb.retries]):
+                    lb_data['options'] = {
+                        'client_timeout': lb.timeout,
+                        'server_timeout': lb.timeout,
+                        'connect_timeout': lb.timeout,
+                        'connect_retries': lb.retries
+                    }
+
                 lb_data['monitor'] = monitor_data
                 job_data['loadBalancers'].append(lb_data)
 
             # Update the worker
+            mnb_data = {}
             status, response = self._send_message(job_data, 'hpcs_response')
             if not status:
                 self._set_error(data, response, session)
@@ -439,9 +491,17 @@ class GearmanClientThread(object):
                         # time
                         pass
                     elif lb.status == 'BUILD':
-                        # Do nothing, stay in BUILD state until floating IP
-                        # assign finishes
-                        pass
+                        # Do nothing if a new device, stay in BUILD state until
+                        # floating IP assign finishes
+                        if len(lbs) > 1:
+                            lb.status = 'ACTIVE'
+                            if lb.id == self.lbid:
+                                # This is the new LB being added to a device.
+                                # We don't have to assign a vip so we can
+                                # notify billing of the LB creation (once the
+                                # DB is updated)
+                                mnb_data["lbid"] = lb.id
+                                mnb_data["tenantid"] = lb.tenantid
                     else:
                         lb.status = 'ACTIVE'
                         lb.errmsg = None
@@ -452,37 +512,60 @@ class GearmanClientThread(object):
                 # Shouldn't hit here, but just to be safe
                 session.commit()
                 return
+            if device.status == 'BUILD' and len(lbs) > 1:
+                device.status = 'ONLINE'
             device_name = device.name
             device_status = device.status
+            counter = session.query(Counters).\
+                filter(Counters.name == 'loadbalancers_updated').first()
+            counter.value += 1
             session.commit()
             if device_status == 'BUILD':
                 submit_vip_job(
                     'ASSIGN', device_name, None
                 )
 
+            # Send the MnB create if needed
+            if "lbid" in mnb_data:
+                update_mnb('lbaas.instance.create',
+                           mnb_data["lbid"],
+                           mnb_data["tenantid"])
+
     def _send_message(self, message, response_name):
-        job_status = self.gearman_client.submit_job(
-            self.host, message, background=False, wait_until_complete=True,
-            max_retries=10, poll_timeout=120.0
-        )
-        if job_status.state == 'UNKNOWN':
-            # Gearman server connection failed
-            LOG.error('Could not talk to gearman server')
-            return False, "System error communicating with load balancer"
-        if job_status.timed_out:
-            # Job timed out
-            LOG.warning(
-                'Gearman timeout talking to {0}'.format(self.host)
-            )
+
+        self.gear_client.waitForServer()
+
+        job = DisconnectJob(self.host, json.dumps(message))
+
+        self.gear_client.submitJob(job)
+
+        pollcount = 0
+        # Would like to make these config file settings
+        while not job.complete and pollcount < POLL_COUNT:
+            sleep(POLL_SLEEP)
+            pollcount += 1
+
+        if job.disconnect:
+            LOG.error('Gearman Job server fail - disconnect')
+            return False, "Gearman Job server fail - "\
+                "disconnect communicating with load balancer"
+
+        # We timed out waiting for the job to finish
+        if not job.complete:
+            LOG.warning('Gearman timeout talking to {0}'.format(self.host))
             return False, "Timeout error communicating with load balancer"
-        LOG.debug(job_status.result)
-        if 'badRequest' in job_status.result:
-            error = job_status.result['badRequest']['validationErrors']
+
+        result = json.loads(job.data[0])
+
+        LOG.debug(result)
+
+        if 'badRequest' in result:
+            error = result['badRequest']['validationErrors']
             return False, error['message']
-        if job_status.result[response_name] == 'FAIL':
+        if result[response_name] == 'FAIL':
             # Worker says 'no'
-            if 'hpcs_error' in job_status.result:
-                error = job_status.result['hpcs_error']
+            if 'hpcs_error' in result:
+                error = result['hpcs_error']
             else:
                 error = 'Load Balancer error'
             LOG.error(
@@ -490,4 +573,4 @@ class GearmanClientThread(object):
             )
             return False, error
         LOG.info('Gearman success from {0}'.format(self.host))
-        return True, job_status.result
+        return True, result
