@@ -1,3 +1,17 @@
+# Copyright 2013 Hewlett-Packard Development Company, L.P.
+#
+# Licensed under the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License. You may obtain
+# a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+# License for the specific language governing permissions and limitations
+# under the License.
+
 import json
 import logging
 
@@ -19,6 +33,9 @@ class FakeOSServices(ServicesBase):
     def service_start(self):
         pass
 
+    def service_reload(self):
+        pass
+
     def write_config(self, config_str):
         pass
 
@@ -39,6 +56,9 @@ class FakeFaultingOSServices(ServicesBase):
     def service_start(self):
         raise Exception("fault")
 
+    def service_reload(self):
+        raise Exception("fault")
+
     def service_restart(self):
         raise Exception("fault")
 
